package com.dsic.gj_erp.controller.zyjh;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.dsic.gj_erp.annotation.AuthFunction;
import com.dsic.gj_erp.bean.ResultBean;
import com.dsic.gj_erp.bean.jhgk.DmYdjh;
import com.dsic.gj_erp.bean.zyjh.Ycldw;
import com.dsic.gj_erp.bean.zyjh.YcldwInfo;
import com.dsic.gj_erp.controller.zyjh.dto.*;
import com.dsic.gj_erp.exception.ServiceException;
import com.dsic.gj_erp.service.zyjh.YcldwInfoService;
import com.dsic.gj_erp.service.zyjh.YcldwService;
import com.dsic.gj_erp.service.zyjh.ZyjhService;
import com.dsic.gj_erp.ws.WsHandler;
import com.dsic.gj_erp.wsclient.Message;
import com.dsic.gj_erp.wsclient.WebSocketClientComponent;
import com.dsic.gj_erp.wsclient.WebSocketService;
import lombok.AllArgsConstructor;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Field;
import java.util.List;

@RestController
@RequestMapping("zyjh")
@AllArgsConstructor
public class ZyjhController {

    private final ZyjhService zyjhService;

    private final YcldwService ycldwService;

    private final YcldwInfoService ycldwInfoService;

    private final WsHandler wsHandler;

    private final WebSocketService webSocketService;
    private final WebSocketClientComponent webSocketClientComponent;

    /**
     * 作业计划执行,派工&反馈
     */
    @PostMapping("zx")
    public ResultBean<?> zx(@RequestBody PgDto dto) {
        if (StrUtil.isEmpty(dto.getZxZtName())) {
            throw new ServiceException(10001,"作业计划执行内容不能为空");
        }
        List<DmYdjh> list=dto.getList();
        try {
            Field field=DmYdjh.class.getDeclaredField(dto.getField());
            if (ObjUtil.isNotEmpty(field)){
                field.setAccessible(true);
                String date=DateUtil.date().toString("yyyy/MM/dd");
                for(DmYdjh dmYdjh:list){
                    field.set(dmYdjh,date);
                    dmYdjh.setJhlx(null);
                    dmYdjh.setZtByZyjhzx(dto.getZxZtName());
                }
            }
            if ("PWFK".equals(dto.getZxZtName())){
                ycldwInfoService.updatePw(dto.getYcldwInfo());
                wsHandler.sendPwComplete((JSONObject) JSONObject.toJSON(dto.getYcldwInfo()));
            }
            if ("HXFK".equals(dto.getZxZtName())){
                ycldwInfoService.updatePw(dto.getYcldwInfo());
                wsHandler.sendHXComplete((JSONObject) JSONObject.toJSON(dto.getYcldwInfo()));
            }
            if ("QGFK".equals(dto.getZxZtName())){
                //无需划线,从理料间获取时才更新垛位信息
                if ("42".equals(dto.getZt())){
                    ycldwInfoService.updatePw(dto.getYcldwInfo());
                }
                wsHandler.sendHXComplete((JSONObject) JSONObject.toJSON(dto.getYcldwInfo()));
                wsHandler.sendQGComplete((JSONObject) JSONObject.toJSON(dto.getYcldwInfo()));
            }
            zyjhService.updateBatchById(list);

            if (dto.getAudioFlag()){
                Message result = Message.result(webSocketClientComponent.getErpId(), "1");
                webSocketService.sendMessage(result.toString());
            }

            return new ResultBean<>();
        } catch (NoSuchFieldException | IllegalAccessException e) {
            e.printStackTrace();
            if (dto.getAudioFlag()){
                Message result = Message.result(webSocketClientComponent.getErpId(), "0");
                try {
                    webSocketService.sendMessage(result.toString());
                } catch (Exception ex) {
                    e.printStackTrace();
                    throw new ServiceException(10001,"执行失败");
                }
            }
            throw new ServiceException(10001,"反馈属性不存在");
        } catch (Exception e) {
            e.printStackTrace();
            throw new ServiceException(10001,"执行失败");
        }
    }
    @PostMapping("zxweb")
    @AuthFunction
    public ResultBean<?> zxweb(@RequestBody PgDto dto,HttpServletRequest request) {
        if (StrUtil.isEmpty(dto.getZxZtName())) {
            throw new ServiceException(10001,"作业计划执行内容不能为空");
        }
        List<DmYdjh> list=dto.getList();

        try {
            Field field=DmYdjh.class.getDeclaredField(dto.getField());
            Field fieldry=DmYdjh.class.getDeclaredField(dto.getFieldry());
            if (ObjUtil.isNotEmpty(field)){
                field.setAccessible(true);
                String date=DateUtil.date().toString("yyyy/MM/dd");
                for(DmYdjh dmYdjh:list){
                    field.set(dmYdjh,date);
                    dmYdjh.setZtByZyjhzx(dto.getZxZtName());

                    if(ObjUtil.isNotEmpty(fieldry)){
                        fieldry.setAccessible(true);
                        fieldry.set(dmYdjh,request.getAttribute("yhms"));
                    }
                }

            }
            zyjhService.updateBatchById(list);
            return new ResultBean<>();
        } catch (NoSuchFieldException | IllegalAccessException e) {
            throw new ServiceException(10001,"属性字段不存在");
        }
    }

    /**
     * 派工单查询
     */
    @PostMapping("pgd")
    public ResultBean<?> pgd(@RequestBody PgdSearch search){
        List<DmYdjh> list = zyjhService.list(Wrappers.<DmYdjh>query()
                //执行查询
                .eq(StrUtil.isNotEmpty(search.getField()),search.getField(),search.getUser())
//                .isNull(StrUtil.isEmpty(search.getField()),search.getField())
                .eq(StrUtil.isNotEmpty(search.getZt()),"zt",search.getZt()).or(StrUtil.isNotEmpty(search.getZtHx()),it->it.eq("zt",search.getZtHx()))
                .eq(StrUtil.isNotEmpty(search.getDcch()),"dcch",search.getDcch())
                .le(StrUtil.isNotEmpty(search.getJhrqField()),search.getJhrqField(), DateUtil.date().toString("yyyy/MM/dd"))
                //跟踪查询
                .eq(StrUtil.isNotEmpty(search.getXqrq()),"xqrq",search.getXqrq())
        );
        return new ResultBean<>(list);
    }


    @PostMapping("pgdweb")
    @AuthFunction
    public ResultBean<?> pgdweb(@RequestBody PgdSearch search){
        List<DmYdjh> list = zyjhService.list(Wrappers.<DmYdjh>query()
            //执行查询
            .eq(StrUtil.isNotEmpty(search.getField()),search.getField(),search.getUser())
            .eq(StrUtil.isNotEmpty(search.getZt()),"zt",search.getZt())
            .eq(StrUtil.isNotEmpty(search.getDcch()),"dc_ch",search.getDcch())
            .le(StrUtil.isNotEmpty(search.getJhrqField()),search.getJhrqField(), DateUtil.date().toString("yyyy/MM/dd"))
            //跟踪查询
            .eq(StrUtil.isNotEmpty(search.getXqrq()),"xqrq",search.getXqrq())
        );
        return new ResultBean<>(list);
    }


    /**
     * 获取上料计划,所有派工单数据整合到月度计划中统一管理,
     * 单独的派工单页面需要进行调整,从月度计划中获取对应工序数据
     */
    @PostMapping("sljh")
    public ResultBean<?> sljh(@RequestBody SljhSearch search){
        String date = DateUtil.date().toString("yyyy/MM/dd");
        if (StrUtil.isAllNotEmpty(search.getSljhrq())){
            date=search.getSljhrq();
        }
        List<DmYdjh> list = zyjhService.list(Wrappers.<DmYdjh>lambdaQuery()
                .le(DmYdjh::getSljhrq, date)
                .eq(StrUtil.isNotEmpty(search.getZt()),DmYdjh::getZt, search.getZt())
                .eq(StrUtil.isNotEmpty(search.getDcch()),DmYdjh::getDcCh, search.getDcch())
                .eq(StrUtil.isNotEmpty(search.getPl()),DmYdjh::getDcPl, search.getPl())
                .eq(StrUtil.isNotEmpty(search.getKw()),DmYdjh::getKw, search.getKw())
        );
        return new ResultBean<>(list);
    }

//    @PostMapping("sljhWeb")
//    public ResultBean<?> sljhWeb(@RequestBody SljhSearch search){
//        String date = DateUtil.date().toString("yyyy/MM/dd");
//        if (StrUtil.isAllNotEmpty(search.getSljhrq())){
//            date=search.getSljhrq();
//        }
//        List<DmYdjh> list = zyjhService.list(Wrappers.<DmYdjh>lambdaQuery()
//            .le(DmYdjh::getSljhrq, date)
//            .eq(StrUtil.isNotEmpty(search.getDcch()),DmYdjh::getZt, search.getZt())
//            .eq(StrUtil.isNotEmpty(search.getDcch()),DmYdjh::getDcCh, search.getDcch())
//            .eq(StrUtil.isNotEmpty(search.getPl()),DmYdjh::getDcPl, search.getPl())
//            .eq(StrUtil.isNotEmpty(search.getKw()),DmYdjh::getKw, search.getKw())
//        );
//        return new ResultBean<>(list);
//    }

    /**
     * 获取预处理垛位信息
     */
    @PostMapping("ycldw")
    public ResultBean<?> ycldw(String type){
        return new ResultBean<>(ycldwService.ycldwList(type));
    }



    @PostMapping("slweb")
    @Transactional(rollbackFor = Exception.class)
    @AuthFunction
    public ResultBean<?> slweb(@RequestBody List<DmYdjh> ydjhs, HttpServletRequest request) {

        for(DmYdjh dmYdjh :ydjhs){
            ShangLiao shangLiao = new ShangLiao();
            dmYdjh.setSlfkrq(DateUtil.format(DateUtil.date(), "yyyy/MM/dd HH:mm:ss"));
            dmYdjh.setSlfkry(  (String) request.getAttribute("yhms"));
            shangLiao.setYdjh(dmYdjh);
           // shangLiao.setCkwpkwp();
           // shangLiao.setPgdId();
            shangLiao.setDwId(dmYdjh.getSlycldw());
            shangLiao.setCeng(dmYdjh.getSlyclcs());
            shangLiao.setCzbh(dmYdjh.getDcCh());
            shangLiao.setPl(dmYdjh.getDcPl());
            shangLiao.setFd(dmYdjh.getDcFd());
            shangLiao.setKw(dmYdjh.getKw());
            shangLiao.setZl(dmYdjh.getZl());
            shangLiao.setTlth(dmYdjh.getTzbh());
           // shangLiao.setWph(dmYdjh.getm);
            shangLiao.setWpgg(dmYdjh.getWpgg());
            shangLiao.setSljhrq(dmYdjh.getSljhrq());
            shangLiao.setQgjhrq(dmYdjh.getSljhrq());
            shangLiao.setBfr( (String) request.getAttribute("yhms"));
            shangLiao.setType(Ycldw.DwTypeEnum.YCL);
            //更新预处理垛位信息
            if (!shangLiao.checkCkwp()){
                //throw new ServiceException(10009,"物品库存不存在,或库存信息错误");
            }
            YcldwInfo info=ycldwInfoService.updateForSl(shangLiao);

            //更新月度计划上料相关字段
            zyjhService.updateForSl(shangLiao);

            //todo 更新库存信息

            wsHandler.sendSlComplete((JSONObject)JSONObject.toJSON(info));
        }





        return new ResultBean<>();
    }
    /**
     * 上料
     */
    @PostMapping("sl")
    @Transactional(rollbackFor = Exception.class)
    public ResultBean<?> sl(@RequestBody ShangLiao sl) {
        if (!sl.checkCkwp()){
            //throw new ServiceException(10009,"物品库存不存在,或库存信息错误");
        }

        //更新预处理垛位信息
        YcldwInfo info=ycldwInfoService.updateForSl(sl);

        //更新月度计划上料相关字段
        zyjhService.updateForSl(sl);

        //todo 更新库存信息

        switch (sl.getType()){
            case YCL:
                wsHandler.sendSlComplete((JSONObject)JSONObject.toJSON(info));
                break;
            case LLJ:
                wsHandler.sendLLComplete((JSONObject)JSONObject.toJSON(info));
                break;
        }

        return new ResultBean<>();
    }

    /**
     * 倒垛
     */
    @PostMapping("daoduo")
    @Transactional(rollbackFor = Exception.class)
    public ResultBean<?> daoduo(@RequestBody DaoDuo dd){
        if (dd.getLastCeng()!=dd.getCeng()||dd.getLastDuo()!=dd.getDuo()){
            int moveY=dd.getLastCeng()-dd.getCeng();
            int y=moveY>0?1:-1;
            for (YcldwInfo info:dd.getMovedList()){
                ycldwInfoService.daoduo(info.getId(),y);
            }
            ycldwInfoService.daoduo(dd.getId(),-moveY);
        }
        return new ResultBean<>();
    }

}
